// Copyright © 2020 Amazon Web Services
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package org.opengroup.osdu.register.provider.aws.pushApi;


import com.amazonaws.services.sns.AmazonSNS;
import com.amazonaws.services.sns.model.ConfirmSubscriptionRequest;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.hash.Hashing;
import org.opengroup.osdu.core.aws.sns.AmazonSNSConfig;
import org.opengroup.osdu.core.common.cryptographic.ISignatureService;
import org.opengroup.osdu.core.common.cryptographic.SignatureServiceException;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.register.provider.aws.config.AwsServiceConfig;
import org.opengroup.osdu.register.utils.AppServiceConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.annotation.RequestScope;


import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.Map;
import java.util.Scanner;

//used  by integration test to validate challenge response logic and confirm subscription
@RestController
@RequestMapping("/awstest")
@RequestScope
@Validated
public class AwsSubscriberTestListenerApi {

    @Autowired
    private AppServiceConfig serviceConfig;
    @Autowired
    private ISignatureService signatureService;
    @Autowired
    private JaxRsDpsLog logger;
    @Inject
    private javax.inject.Provider<DpsHeaders> headersProvider;

    @Autowired
    private AwsServiceConfig awsConfig;

    public AmazonSNS snsClient;

    @PostMapping("/aws/challenge/{count}")
    public ResponseEntity process(HttpServletRequest request, HttpServletResponse response) throws IOException {

        String messagetype = request.getHeader("x-amz-sns-message-type");
        if(messagetype!=null) {
            AmazonSNSConfig snsConfig = new AmazonSNSConfig(awsConfig.amazonRegion);
            snsClient = snsConfig.AmazonSNS();
            Scanner scan = new Scanner(request.getInputStream());
            StringBuilder builder = new StringBuilder();
            while (scan.hasNextLine()) {
                builder.append(scan.nextLine());
            }
            InputStream bytes = new ByteArrayInputStream(builder.toString().getBytes());
            Map<String, String> messageMap = new ObjectMapper().readValue(bytes, Map.class);
            logger.info("Type="+messageMap.get("Type"));
            logger.info("Message="+messageMap.get("Message"));
            logger.info("TopicArn="+messageMap.get("TopicArn"));
            if (messagetype.equals("SubscriptionConfirmation")) {
                logger.info("Subscription Confirmation received=" + messageMap.get("Type"));
                String topicarn = messageMap.get("TopicArn");
                String token = messageMap.get("Token");
                ConfirmSubscriptionRequest confirmReq = new ConfirmSubscriptionRequest()
                        .withTopicArn(topicarn)
                        .withToken(token);
                snsClient.confirmSubscription(confirmReq);
                logger.info("Subscription confirmed");
                return new ResponseEntity<>(HttpStatus.OK);
            }
            if (messagetype.equals("Notification")) {
                logger.info("Message="+messageMap.get("Message"));
                try {
                    String hmac= request.getParameter("hmac");
                    signatureService.verifyHmacSignature(hmac, this.serviceConfig.getSubscriberSecret());
                } catch (SignatureServiceException e) {
                    return new ResponseEntity<>("Authorization signature validation Failed", HttpStatus.BAD_REQUEST);
                }
                logger.info("Sending acknowledgement from hmac endpoint");
                return new ResponseEntity<>(HttpStatus.OK);
            }
            if (messagetype.equals("UnsubscribeConfirmation")) {
                logger.info("Unsubscribe Confirmation received");
            }
        }else
        {
            logger.info("Subscription Notification Message received");
            try {
                Scanner scan = new Scanner(request.getInputStream());
                StringBuilder builder = new StringBuilder();
                while (scan.hasNextLine()) {
                    builder.append(scan.nextLine());
                }
                logger.info("Message Received is="+builder.toString());

                String hmac= request.getParameter("hmac");
                signatureService.verifyHmacSignature(hmac, this.serviceConfig.getSubscriberSecret());
            } catch (SignatureServiceException e) {
                return new ResponseEntity<>("Authorization signature validation Failed", HttpStatus.BAD_REQUEST);
            }
            logger.info("Sending acknowledgement from hmac endpoint");
            return new ResponseEntity<>(HttpStatus.OK);

        }
        return new ResponseEntity<>(HttpStatus.OK);
    }



    @GetMapping("/aws/challenge/{count}")
    public ResponseEntity<?> testCrc(@RequestParam("crc") @NotBlank String crc, @RequestParam("hmac") @NotBlank String hmac) {
        try {
            signatureService.verifyHmacSignature(hmac, this.serviceConfig.getSubscriberSecret());
        } catch (SignatureServiceException e) {
            return new ResponseEntity<>("Authorization signature validation Failed", HttpStatus.BAD_REQUEST);
        }
        logger.info("Signature verified and sending response");
        // Use the secret you send to the subscriber registration create request
        return getResponse(crc, this.serviceConfig.getSubscriberSecret());
    }

    private ResponseEntity<ChallengeResponse> getResponse(String crc, String secretString) {
        String response = secretString + crc;
        response = Hashing.sha256()
                .hashString(response, StandardCharsets.UTF_8)
                .toString();
        response = Base64.getEncoder().encodeToString(response.getBytes());
        ChallengeResponse cr = new ChallengeResponse();
        cr.responseHash = response;
        return new ResponseEntity<>(cr, HttpStatus.OK);
    }

    class ChallengeResponse {
        public String responseHash = "";
    }



}
