/*
  Copyright 2021 Google LLC
  Copyright 2021 EPAM Systems, Inc

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package org.opengroup.osdu.register.provider.gcp.ddms;

import com.google.gson.Gson;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.common.model.tenant.TenantInfo;
import org.opengroup.osdu.core.gcp.osm.model.Destination;
import org.opengroup.osdu.core.gcp.osm.model.Kind;
import org.opengroup.osdu.core.gcp.osm.model.Namespace;
import org.opengroup.osdu.core.gcp.osm.model.query.GetQuery;
import org.opengroup.osdu.core.gcp.osm.service.Context;
import org.opengroup.osdu.core.gcp.osm.service.Transaction;
import org.opengroup.osdu.core.gcp.osm.translate.ExceptionClassifier;
import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.provider.interfaces.ddms.IDdmsRepository;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.concurrent.locks.ReentrantLock;

import static org.opengroup.osdu.core.gcp.osm.model.where.predicate.Eq.eq;
import static org.springframework.beans.factory.config.BeanDefinition.SCOPE_SINGLETON;

@Slf4j
@Repository
@Scope(SCOPE_SINGLETON)
@ConditionalOnProperty(name = "osmDriver")
@RequiredArgsConstructor
public class OsmDdmsRepository implements IDdmsRepository {

    private static final Gson gson = new Gson();
    private static final String CONFLICT_REASON = "Conflict";

    private final Context context;
    private final TenantInfo tenantInfo;
    ReentrantLock ddmsDeleteLock = new ReentrantLock();

    private static final Namespace NAMESPACE = new Namespace("DE");
    private static final Kind KIND = new Kind("DDMS");

    //queries addressed to tenant specific GCP project and namespace and to a certain kind.
    private Destination getDestination() {
        return Destination.builder().partitionId(tenantInfo.getDataPartitionId()).namespace(NAMESPACE).kind(KIND).build();
    }

    @Override
    public Ddms create(Ddms ddms) {
        Transaction txn = context.beginTransaction(getDestination());
        try {
            if (context.getOne(new GetQuery<>(Ddms.class, getDestination(), eq("id", ddms.getId()))) != null) {
                throw new AppException(409, CONFLICT_REASON, "A DDMS already exists with the same id");
            }
            context.create(ddms, getDestination());
            txn.commitIfActive();
        } catch (AppException e) {
            throw e;
        } catch (Exception ex) {
            ExceptionClassifier.ExceptionClassification classification = context.classifyException(ex);
            switch (classification) {
                case ALREADY_EXISTS:
                    throw new AppException(409, CONFLICT_REASON, "A DDMS already exists with the same id");
                case CONFLICT:
                    throw new AppException(409, CONFLICT_REASON, "ABORTED due to conflict");
                default:
                    throw ex;
            }
        } finally {
            txn.rollbackIfActive();
        }
        return ddms;
    }

    @Override
    public Ddms get(String id) {
        Ddms ddms = context.getOne(new GetQuery<>(Ddms.class, getDestination(), eq("id", id)));
        if (ddms == null)
            throw new AppException(404, "Not found", String.format("DDMS with id %s does not exist.", id));
        return ddms;
    }

    @Override
    public List<Ddms> query(String type) {
        GetQuery<Ddms> query = new GetQuery<>(Ddms.class, getDestination(), eq("interfaces.entityType", type));
        return context.getResultsAsList(query);
    }

    @Override
    public boolean delete(String id) {
        boolean output = false;

        Transaction txn = context.beginTransaction(getDestination());
        try {
            ddmsDeleteLock.lock();
            if (context.getOne(new GetQuery<>(Ddms.class, getDestination(), eq("id", id))) != null) {
                context.deleteById(Ddms.class, getDestination(), id);
                txn.commitIfActive();
                output = true;
            }
        } catch (Exception ex) {
            ExceptionClassifier.ExceptionClassification classification = context.classifyException(ex);
            switch (classification) {
                case ALREADY_EXISTS:
                    throw new AppException(409, CONFLICT_REASON, "Another request is trying to delete the same DDMS.");
                case CONFLICT:
                    throw new AppException(409, CONFLICT_REASON, "ABORTED due to conflict");
                default:
                    throw ex;
            }
        } finally {
            ddmsDeleteLock.unlock();
            txn.rollbackIfActive();
        }
        return output;
    }
}
