/*
  Copyright 2021 Google LLC
  Copyright 2021 EPAM Systems, Inc

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */

package org.opengroup.osdu.register.provider.reference.mapper;

import java.sql.Timestamp;
import lombok.RequiredArgsConstructor;
import org.opengroup.osdu.core.common.provider.interfaces.IKmsClient;
import org.opengroup.osdu.register.provider.reference.model.document.SubscriptionDocument;
import org.opengroup.osdu.register.subscriber.model.GsaSecret;
import org.opengroup.osdu.register.subscriber.model.GsaSecretValue;
import org.opengroup.osdu.register.subscriber.model.HmacSecret;
import org.opengroup.osdu.register.subscriber.model.Secret;
import org.opengroup.osdu.register.subscriber.model.Subscription;
import org.opengroup.osdu.register.utils.Constants;
import org.springframework.stereotype.Component;

@Component
@RequiredArgsConstructor
public class SubscriptionMapper {

  private static final String ENCRYPTION_PREFIX = "this.kmsClient.encryptString ";

  private final IKmsClient kmsClient;

  public Subscription toSubscription(SubscriptionDocument document) {
    if (document == null) {
      return null;
    }

    Subscription subscription = new Subscription();
    subscription.setId(document.getId());
    subscription.setName(document.getName());
    subscription.setDescription(document.getDescription());
    subscription.setTopic(document.getTopic());
    subscription.setPushEndpoint(document.getPushEndpoint());
    subscription.setCreatedBy(document.getCreatedBy());
    subscription.setCreatedOnEpoch(Timestamp.from(document.getCreatedOn().toInstant()));
    subscription.setNotificationId(document.getNotificationId());

    String decryptedSecret = document.getSecretValue().replace(ENCRYPTION_PREFIX, "");
    Secret secret;
    if (document.getSecretType().equals(Constants.GSASecret)) {
      GsaSecret gsaSecret = new GsaSecret();
      String[] splitSecret = decryptedSecret.split("`");
      gsaSecret.setValue(new GsaSecretValue(splitSecret[0], splitSecret[1]));
      secret = gsaSecret;
    } else {
      HmacSecret hmacSecret = new HmacSecret();
      hmacSecret.setValue(decryptedSecret);
      secret = hmacSecret;
    }

    subscription.setSecret(secret);

    return subscription;
  }

  public SubscriptionDocument toDocument(Subscription subscription) {
    String encryptedSecret = ENCRYPTION_PREFIX + subscription.getSecret().toString();
    return SubscriptionDocument.builder()
        .id(subscription.getId())
        .name(subscription.getName())
        .description(subscription.getDescription())
        .topic(subscription.getTopic())
        .pushEndpoint(subscription.getPushEndpoint())
        .createdBy(subscription.getCreatedBy())
        .createdOn(subscription.getCreatedOnEpoch())
        .notificationId(subscription.getNotificationId())
        .secretType(subscription.getSecret().getSecretType())
        .secretValue(encryptedSecret)
        .build();
  }

  public SubscriptionDocument toUpdatedDocument(Subscription subscription, Secret secret) {
    String secretType = secret.getSecretType();
    String encryptedSecret = ENCRYPTION_PREFIX + secret.toString();
    return SubscriptionDocument.builder()
        .id(subscription.getId())
        .name(subscription.getName())
        .description(subscription.getDescription())
        .topic(subscription.getTopic())
        .pushEndpoint(subscription.getPushEndpoint())
        .createdBy(subscription.getCreatedBy())
        .createdOn(subscription.getCreatedOnEpoch())
        .notificationId(subscription.getNotificationId())
        .secretType(secretType)
        .secretValue(encryptedSecret)
        .build();
  }
}
