/*
  Copyright 2021 Google LLC
  Copyright 2021 EPAM Systems, Inc

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */

package org.opengroup.osdu.register.provider.reference.action;

import static org.junit.Assert.assertEquals;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;
import static org.powermock.api.mockito.PowerMockito.doReturn;

import java.util.Collections;
import java.util.Optional;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Spy;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.register.action.model.Action;
import org.opengroup.osdu.register.provider.reference.mapper.ActionMapper;
import org.opengroup.osdu.register.provider.reference.model.document.ActionDocument;
import org.opengroup.osdu.register.provider.reference.repository.mongo.ActionDocumentRepository;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

@RunWith(PowerMockRunner.class)
@PrepareForTest(ReferenceActionRepository.class)
public class ReferenceActionRepositoryTest {

  private static final String CHECKSUM = "checksum";

  private ActionDocumentRepository actionDocumentRepository = PowerMockito.mock(
      ActionDocumentRepository.class);
  private ActionMapper actionMapper = PowerMockito.mock(ActionMapper.class);

  @Spy
  private ReferenceActionRepository referenceActionRepository = new ReferenceActionRepository(
      actionDocumentRepository, actionMapper);

  private Action action;
  private ActionDocument actionDocument;

  @Before
  public void setUp() {
    action = new Action();
    action.setId("actionId");

    actionDocument = new ActionDocument();
    actionDocument.setId("documentId");
    actionDocument.setCheckSum(CHECKSUM);
  }

  @Test
  public void should_createAction_when_actionDoesNotExist() throws Exception {
    when(actionDocumentRepository.existsById(action.getId())).thenReturn(false);
    doReturn(CHECKSUM).when(referenceActionRepository, "getActionChecksum", action);
    when(actionMapper.toDocument(action, CHECKSUM)).thenReturn(actionDocument);
    when(actionDocumentRepository.save(actionDocument)).thenReturn(actionDocument);
    when(actionMapper.toAction(actionDocument)).thenReturn(action);

    assertEquals(action, referenceActionRepository.createAction(action));
  }

  @Test(expected = AppException.class)
  public void should_throwException_when_creatingActionWhichExists() {
    when(actionDocumentRepository.existsById(action.getId())).thenReturn(true);

    this.referenceActionRepository.createAction(action);
  }

  @Test
  public void should_getAction_when_actionExistsAndChecksumIsValid() throws Exception {
    when(actionDocumentRepository.findById(action.getId())).thenReturn(
        Optional.of(actionDocument));
    when(actionMapper.toAction(actionDocument)).thenReturn(action);
    doReturn(CHECKSUM).when(referenceActionRepository, "getActionChecksum", action);

    assertEquals(action, referenceActionRepository.get(action.getId()));
  }

  @Test(expected = AppException.class)
  public void should_throwException_when_gettingActionWhichExistsAndChecksumIsInvalid() {
    when(actionDocumentRepository.findById(action.getId())).thenReturn(
        Optional.of(actionDocument));
    when(actionMapper.toAction(actionDocument)).thenReturn(action);

    referenceActionRepository.get(action.getId());
  }

  @Test(expected = AppException.class)
  public void should_throwException_when_gettingActionWhichDoesNotExist() {
    referenceActionRepository.get(action.getId());
  }

  @Test
  public void should_deleteAction_when_actionExists() {
    when(actionDocumentRepository.existsById(any())).thenReturn(true);

    assertEquals(true, referenceActionRepository.delete(any()));
  }

  @Test
  public void shouldNot_deleteAction_when_actionDoesNotExist() {
    when(actionDocumentRepository.existsById(any())).thenReturn(false);

    assertEquals(false, referenceActionRepository.delete(any()));
  }

  @Test
  public void should_getAllActions_when_actionChecksumIsValid() throws Exception {
    when(actionDocumentRepository.findAll()).thenReturn(Collections.singletonList(actionDocument));
    when(actionMapper.toAction(actionDocument)).thenReturn(action);
    doReturn(CHECKSUM).when(referenceActionRepository, "getActionChecksum", action);

    assertEquals(Collections.singletonList(action), referenceActionRepository.getAllActions());
  }

  @Test(expected = AppException.class)
  public void should_throwException_when_gettingAllActionsAndChecksumIsInvalid() {
    when(actionDocumentRepository.findAll()).thenReturn(Collections.singletonList(actionDocument));
    when(actionMapper.toAction(actionDocument)).thenReturn(action);

    referenceActionRepository.getAllActions();
  }
}
